#!/bin/bash
# This script creates an Aegir platform, from the download up to the front-end.

#****************************************
# Variables
#****************************************

# The name of this file.
THIS_FILE=$(basename $0)

# The full path of this file.
THIS_PATH=$(dirname $(pwd)/$0)

# Today's date and time.
DATE="$(date +%Y).$(date +%m).$(date +%d)"
TIME="$(date +%H).$(date +%M).$(date +%S)"

# Requirements
REQ_USER="aegir"
REQ_DRUSH_VERSION="4.5"
REQ_DRUSH_MAKE_VERSION="2.3"

# Makefile specifications.
MAKEFILE_API="2"      # drush make API
MAKEFILE_CORE="6.x"   # Drupal core

# Default platform name.
PLATFORM_NAME="drupal-6"

# Default inclusions.
INCLUSIONS="modules themes libraries"

# System
DRUSH="drush"                           # Path to the command "drush".
HOSTMASTER="hostmaster"                 # To use like "drush @hostmaster ...".
INCLUDIR="$THIS_PATH/includes"          # Directory where to find the makefiles to include.
PATH_PLATFORMS="/var/aegir/platforms"   # Directory where to create platforms.
TMPDIR="/tmp"                           # Temporary directory where to build the stub makefile.

# Files
FAVICON_FILE="favicon.ico"
README_FILE="README.txt"

# Warning detection.
WARNING=0

#****************************************
# Arguments
#****************************************

# Processing arguments and defining variables.
while getopts ":a:d:fh:i:n:p:s:" opt; do
  case $opt in

    a) # -a htpasswd-file
      HTPASSWD="$OPTARG"
      ;;

    d) # -d drush-command
      DRUSH="$OPTARG"
      ;;

    f)
      FORCE_COMPLETE=1
      ;;

    h) # -h hostmaster-instance-name
      HOSTMASTER="$OPTARG"
      ;;

    i) # -i inclusions

      # Replacing upper case by lower case.
      INCLUSIONS=$(echo $OPTARG | tr '[:upper:]' '[:lower:]')

      # Replacing comas by space if a coma separated list is given.
      INCLUSIONS=$(echo $INCLUSIONS | sed 's/,/ /g')

      ;;

    n) # -n platform-name
      PLATFORM_NAME="$OPTARG"
      ;;

    p) # -p path-to-platforms
      PATH_PLATFORMS="$OPTARG"
      ;;

    s) # -s platform-state
      PLATFORM_STATE="$OPTARG"
      ;;

    \?)
      echo "ERROR: Invalid option: -$OPTARG"
      exit 1
      ;;

    :)
      echo "ERROR: Option -$OPTARG requires an argument."
      exit 1
      ;;

  esac
done

#****************************************
# Checking requirements
#****************************************

# User

USER=$(id -nu)
echo "Current user = '$USER'"

if [ "$USER" != "$REQ_USER" ] ; then
  echo "ERROR: You must run '$THIS_FILE' as user '$REQ_USER'."
  exit 1
fi

# Drush version

DRUSH_VERSION=$($DRUSH --version | awk '{print $NF}')
echo "Current 'drush' version = '$DRUSH_VERSION'"

if [ $(echo "$DRUSH_VERSION < $REQ_DRUSH_VERSION" | bc) -eq 1 ]; then
  echo "ERROR: 'Drush' version must be '$REQ_DRUSH_VERSION' or higher."
  exit 1
fi

# Drush make version

DRUSH_MAKE_VERSION=$($DRUSH make --version | awk '{print $NF}')
echo "Current 'drush make' version = '$DRUSH_MAKE_VERSION'"

if [ $(echo "$DRUSH_MAKE_VERSION < $REQ_DRUSH_MAKE_VERSION" | bc) -eq 1 ]; then
  echo "ERROR: 'Drush make' version must be '$REQ_DRUSH_MAKE_VERSION' or higher."
  exit 1
fi

#****************************************
# Building platform information
#****************************************

# The name of the platform directory without any path.
if [ "$PLATFORM_STATE" = "" ]; then
  PLATFORM_DIRECTORY="$PLATFORM_NAME-build-$DATE"
else
  PLATFORM_DIRECTORY="$PLATFORM_NAME-build-$DATE-$PLATFORM_STATE"
fi
echo "Platform name = '$PLATFORM_DIRECTORY'"

# Absolute path to the platform.
PATH_PLATFORM="$PATH_PLATFORMS/$PLATFORM_DIRECTORY"
echo "Platform path = '$PATH_PLATFORM'"

#****************************************
# Building stub makefile
#****************************************

# The stub makefile with its full path.
MAKEFILE="$TMPDIR/kplatforms-stub_$PLATFORM_DIRECTORY-$TIME.make"
echo "Stub makefile = '$MAKEFILE'"

# Including specifications while creating file.
echo "api = $MAKEFILE_API" >$MAKEFILE
echo "core = $MAKEFILE_CORE" >>$MAKEFILE

# Including core makefile.
echo "includes[core] = $INCLUDIR/core.make" >>$MAKEFILE

# Including other makefiles.
echo "Inclusions = '$INCLUSIONS'"
for INCLUSION in $INCLUSIONS; do

  # The makefile with its full path.
  INCLUSION_MAKEFILE="$INCLUDIR/$INCLUSION.make"
  echo "Including makefile '$INCLUSION_MAKEFILE'"

  # Checking if we can read the makefile.
  if [ ! -r "$INCLUSION_MAKEFILE" ]; then
    echo "ERROR: The makefile '$INCLUSION_MAKEFILE' cannot be read."
    rm $MAKEFILE
    exit 1
  fi

  # Including the makefile.
  echo "includes[$INCLUSION] = $INCLUSION_MAKEFILE" >>$MAKEFILE

done

#****************************************
# Drush commands
#****************************************

# Using "drush --force-complete".
if ! [ -z $FORCE_COMPLETE ]; then
  DRUSH="$DRUSH --force-complete"
fi

echo "Drush command = '$DRUSH'"

# Creating platform.
$DRUSH make $MAKEFILE $PATH_PLATFORM && \
  $DRUSH --root=$PATH_PLATFORM provision-save @platform_$PLATFORM_DIRECTORY --context_type=platform --makefile=$MAKEFILE && \
  $DRUSH @$HOSTMASTER hosting-import @platform_$PLATFORM_DIRECTORY

# Test exit code.
if [ $? -eq 0 ]; then
  echo "OK: Platform created successfully in '$PATH_PLATFORM'."
else
  echo "ERROR: Platform not created successfully."
  exit 1
fi

#****************************************
# Access restricted by users (htauth)
#****************************************

if [ "$HTPASSWD" != "" ]; then

  # Checking if we can read the htpasswd file.
  if [ ! -r "$HTPASSWD" ]; then
    echo "WARNING: The htpasswd file cannot be read."
    echo "Use: 'htpasswd -b -c $HTPASSWD username password'."
    WARNING=`expr $WARNING + 1`
  fi

  # Defining variables.
  HTACCESS_FILE="$PATH_PLATFORM/.htaccess"
  HTACCESS_FILE_HEAD="$PATH_PLATFORM/.htaccess.head"
  HTACCESS_FILE_TMP="$PATH_PLATFORM/.htaccess.tmp"

  # Editing htaccess file.
  echo "# Access restricted by users" >$HTACCESS_FILE_HEAD && \
    echo "AuthUserFile $HTPASSWD" >>$HTACCESS_FILE_HEAD && \
    echo "AuthName 'Authorized people only'" >>$HTACCESS_FILE_HEAD && \
    echo "AuthType Basic" >>$HTACCESS_FILE_HEAD && \
    echo "require valid-user" >>$HTACCESS_FILE_HEAD && \
    echo "" >>$HTACCESS_FILE_HEAD && \
    cat $HTACCESS_FILE_HEAD $HTACCESS_FILE >$HTACCESS_FILE_TMP && \
    mv $HTACCESS_FILE_TMP $HTACCESS_FILE && \
    rm $HTACCESS_FILE_HEAD

  # Test exit code.
  if [ $? -eq 0 ]; then
    echo "OK: Sites on this platform restricted successfully with '$HTPASSWD'."
  else
    echo "ERROR: Sites on this platform not restricted successfully."
    exit 1
  fi

fi

#****************************************
# Platform improvements
#****************************************

# Favicon, for flavour.
cp $THIS_PATH/$FAVICON_FILE $PATH_PLATFORM/
if [ $? -eq 0 ]; then
  echo "OK: Favicon file copied successfully."
else
  echo "WARNING: Favicon file not copied successfully."
  WARNING=`expr $WARNING + 1`
fi

# Making changes as per the inclusions.
for INCLUSION in $INCLUSIONS; do
  case $INCLUSION in

    libraries)

      # TCPDF library for the Drupal contrib module "Printer, e-mail and PDF versions" (print).
      # Directories non-writable permissions.

        # Web server group:
        # Use www-data for Debian and Ubuntu.
        # Use apache for RedHat and CentOS.
        WEBSERVER_GRP="www-data"

        # TODO add test for RedHat
        if [ -x /usr/bin/lsb_release ]; then
          OS=$(lsb_release -i | cut -f 2)
          if [ "$OS" = "CentOS" ]; then
            WEBSERVER_GRP=apache
          fi
        fi

        # Modifying permissions and group.
        chmod g+w $PATH_PLATFORM/sites/all/libraries/tcpdf/cache $PATH_PLATFORM/sites/all/libraries/tcpdf/images && \
          chgrp $WEBSERVER_GRP $PATH_PLATFORM/sites/all/libraries/tcpdf/cache $PATH_PLATFORM/sites/all/libraries/tcpdf/images

        # Test exit code.
        if [ $? -eq 0 ]; then
          echo "OK: TCPDF chmod and chgrp."
        else
          echo "WARNING: TCPDF chmod and chgrp not done successfully."
          WARNING=`expr $WARNING + 1`
        fi

    ;;   # End of libraries.

  esac
done

#****************************************
# Exiting
#****************************************

if [ $WARNING -eq 0 ]; then
  echo "OK: '$THIS_FILE' terminated successfully."
  exit 0
elif [ $WARNING -eq 1 ]; then
  echo "WARNING: '$THIS_FILE' terminated with $WARNING warning."
  exit $WARNING
else
  echo "WARNING: '$THIS_FILE' terminated with $WARNING warnings."
  exit $WARNING
fi

#****************************************
# End
#****************************************
